     # -*- coding: utf-8 -*-
"""
Created on Tue Feb 14 15:59:11 2017

@author: af5u13
"""

import visr
import rcl
import rrl
import objectmodel as om
import panning
import numpy as np;
import matplotlib.pyplot as plt

from vbap_l1_l2_panner import VbapL1L2Panner
from allrad_gain_calculator import AllRadGainCalculator

from helper.vectorFunctions import normalise, angleDifference
from helper.baseTrigFunctions import cart2sph, sph2cart, deg2rad, rad2deg
from helper.panningVectorMetrics import rv, re

bs = 128
samplingFrequency = 48000
ctxt = visr.SignalFlowContext( bs, samplingFrequency)

numBlocks = 360

numObjects = 1

lc = panning.LoudspeakerArray( '../data/bs2051-4+5+0.xml' )
lcMpeg = panning.LoudspeakerArray( '../data/bs2051-4+5+0_mpeg-h_downmix.xml' )
regularAllRadArray = panning.LoudspeakerArray( '../data/t-design_order11_n70.xml' )

numLsp = lc.numberOfRegularLoudspeakers

hoaOrder = 3
#maxReWeights = maxReWeights3D( hoaOrder )
maxReWeights = None

vbapPanner = rcl.PanningCalculator( ctxt, 'calc', None,
                             numberOfObjects = numObjects,
                             arrayConfig = lc,
                             )

mpegPanner = rcl.PanningCalculator( ctxt, 'calc', None,
                             numberOfObjects = numObjects,
                             arrayConfig = lcMpeg,
                             )


l1l2Panner = VbapL1L2Panner( ctxt, 'calc', None,
                       numObjects = numObjects,
                       lspArray = lc )

allRadPanner = AllRadGainCalculator( ctxt, 'calc', None,
                                numberOfObjects=numObjects,
                                regularArrayConfig=regularAllRadArray,
                                realArrayConfig = lc,
                                hoaOrder = hoaOrder,
                                orderWeights = maxReWeights )

flowVbap = rrl.AudioSignalFlow( vbapPanner )
flowMpeg = rrl.AudioSignalFlow( mpegPanner )
flowL1L2 = rrl.AudioSignalFlow( l1l2Panner )
flowAllRad = rrl.AudioSignalFlow( allRadPanner )

flows = [ flowVbap, flowMpeg, flowL1L2, flowAllRad ]

paramInputVbap = flowVbap.parameterReceivePort('objectVectorInput')
paramOutputVbap = flowVbap.parameterSendPort('gainOutput')

paramInputMpeg = flowMpeg.parameterReceivePort('objectVectorInput')
paramOutputMpeg = flowMpeg.parameterSendPort('gainOutput')

paramInputL1L2 = flowL1L2.parameterReceivePort('objects')
paramOutputL1L2 = flowL1L2.parameterSendPort('gains')

paramInputAllRad = flowAllRad.parameterReceivePort('objects')
paramOutputAllRad = flowAllRad.parameterSendPort('gains')

paramInputs = [paramInputVbap, paramInputMpeg, paramInputL1L2, paramInputAllRad ]
paramOutputs = [paramOutputVbap, paramOutputMpeg, paramOutputL1L2, paramOutputAllRad ]

az = np.linspace( 0.0, 2*np.pi, numBlocks )
el = 10.0*np.pi/180.0

gainsVbap = np.zeros( (numBlocks, numLsp) )
gainsMpeg = np.zeros( (numBlocks, numLsp) )
gainsL1L2 = np.zeros( (numBlocks, numLsp) )
gainsAllRad = np.zeros( (numBlocks, numLsp) )

gains = [gainsVbap, gainsMpeg, gainsL1L2, gainsAllRad ]

recGrid = sph2cart( az, el, 1.0 )

for bi in range(0,numBlocks):
    ps1 = om.PointSource(0)
    ps1.position = recGrid[bi,:]
    ps1.channels = [0]; ps1.level=1.0

    for inp in paramInputs:
        inp.data().set( [ps1] )
        inp.swapBuffers()

    for idx, flow in enumerate( flows ):
        flow.process()
        gains[idx][bi,:] = np.array(paramOutputs[idx].data())[:,0]

plt.figure()
for idx in range(0,2):
    plt.plot( 180/np.pi*az, gainsL1L2[:,idx], 'bo:', 180/np.pi*az, gainsAllRad[:,idx], 'm.-' )


numAlgorithms = len( flows )

L = lc.positions().T

pRes = np.zeros( (numAlgorithms, numBlocks, 3) ) # identical to rvVec
pResNormalised = np.zeros( (numAlgorithms, numBlocks, 3) ) # identical to rvVec
gainL1Norm = np.zeros( (numAlgorithms, numBlocks) )
gainL2Norm = np.zeros( (numAlgorithms, numBlocks) )
rvVec = np.zeros( (numAlgorithms, 3, numBlocks) )
reVec = np.zeros( (numAlgorithms, 3, numBlocks) )
rvDirVec = np.zeros( (numAlgorithms, 3, numBlocks) )
reDirVec = np.zeros( (numAlgorithms, 3, numBlocks) )
rvMag = np.zeros( (numAlgorithms, numBlocks) )
reMag = np.zeros( (numAlgorithms, numBlocks) )
rvDirDiff = np.zeros( (numAlgorithms, numBlocks) )
reDirDiff = np.zeros( (numAlgorithms, numBlocks) )
numActiveLoudspeakers = np.zeros( (numAlgorithms, numBlocks), np.int )

for algIdx in range( numAlgorithms ):
    pRes[algIdx,...]  = gains[algIdx] @ L.T
    gainL1Norm[algIdx,...] = np.linalg.norm( gains[algIdx], ord=1,axis=-1)
    gainL2Norm[algIdx,...] = np.linalg.norm( gains[algIdx], ord=2,axis=-1)
    rvVec[algIdx,...], rvDirVec[algIdx,...], rvMag[algIdx,...] = rv( gains[algIdx], L )
    reVec[algIdx,...], reDirVec[algIdx,...], reMag[algIdx,...] = re( gains[algIdx], L)
    rvDirDiff[algIdx,...] = rad2deg(angleDifference( rvDirVec[algIdx,...].T, recGrid ))
    reDirDiff[algIdx,...] = rad2deg(angleDifference( reDirVec[algIdx,...].T, recGrid ))
    numActiveLoudspeakers[algIdx,...] = np.sum( np.abs( gains[algIdx] ) >= 1e-6, axis = -1 )

plt.figure()
plt.plot( rad2deg( az ), gainL1Norm[0,:], 'bo-', rad2deg( az ), gainL1Norm[1,:], 'gs-', rad2deg( az ), gainL1Norm[2,:], 'rx-', rad2deg( az ), gainL1Norm[3,:], 'm.-' )
plt.legend( ('VBAP', 'VBAP MPEG-H', 'VBAP L1L2', 'AllRAD' ) )
plt.title( 'l1 norms (sound pressure)' )

plt.figure()
plt.plot( rad2deg( az ), gainL2Norm[0,:], 'bo-', rad2deg( az ), gainL2Norm[1,:], 'gs-', rad2deg( az ), gainL2Norm[2,:], 'rx-', rad2deg( az ), gainL2Norm[3,:], 'm.-' )
plt.legend( ('VBAP', 'VBAP MPEG-H', 'VBAP L1L2', 'AllRAD' ) )
plt.title( 'l2 norms' )

plt.figure()
plt.plot( rad2deg( az ), rvDirDiff[0,:], 'bo-', rad2deg( az ), rvDirDiff[1,:], 'gs-', rad2deg( az ), rvDirDiff[2,:], 'rx-', rad2deg( az ), rvDirDiff[3,:], 'm.-' )
plt.legend( ('VBAP', 'VBAP MPEG-H', 'VBAP L1L2', 'AllRAD' ) )
plt.title( 'rv direction difference (degree)' )

plt.figure()
plt.plot( rad2deg( az ), reDirDiff[0,:], 'bo-', rad2deg( az ), reDirDiff[1,:], 'gs-', rad2deg( az ), reDirDiff[2,:], 'rx-', rad2deg( az ), reDirDiff[3,:], 'm.-' )
plt.legend( ('VBAP', 'VBAP MPEG-H', 'VBAP L1L2', 'AllRAD' ) )
plt.title( 're direction difference (degree)' )

plt.figure()
plt.plot( rad2deg( az ), numActiveLoudspeakers[0,:], 'bo-', rad2deg( az ), numActiveLoudspeakers[1,:], 'gs-', rad2deg( az ), numActiveLoudspeakers[2,:], 'rx-', rad2deg( az ), numActiveLoudspeakers[3,:], 'm.-' )
plt.legend( ('VBAP', 'VBAP MPEG-H', 'VBAP L1L2', 'AllRAD' ) )
plt.title( 'Number of active loudspeakers' )

velVecMag = np.linalg.norm( pRes, ord=2, axis = -1 )
plt.figure()
plt.plot( rad2deg( az ), velVecMag[0,:], 'bo-', rad2deg( az ), velVecMag[1,:], 'gs-', rad2deg( az ), velVecMag[2,:], 'rx-', rad2deg( az ), velVecMag[3,:], 'm.-' )
plt.legend( ('VBAP', 'VBAP MPEG-H', 'VBAP L1L2', 'AllRAD' ) )
plt.title( 'Raw velocity vector length (for L2-normalised gains)' )
