# -*- coding: utf-8 -*-
"""
Created on Sun Jan 21 17:03:35 2018

@author: af5u13
"""

import panning
import visr
import rcl
import rrl
import efl
import objectmodel as om

import os
import sys
import numpy as np
import matplotlib.pyplot as plt

sys.path.append( 'c:/local/s3a_git/high_order_optimal_panning\code/python' )

from helper.real_sph_harmonics import allSphHarmRealACN
from helper.baseTrigFunctions import cart2sph # ,sph2cart , deg2rad, rad2deg

configBasePath = '../data/'

regConfigFile = '../data/t-design_order11_n70.xml'

ambiOrder = 3

regConfig = panning.LoudspeakerArray( regConfigFile )

az, el, r = cart2sph( regConfig.positions() )

Y = allSphHarmRealACN( ambiOrder, 0.5*np.pi-el, az, dtype = regConfig.positions().dtype )

np.savetxt( '../data/t-design_decoder_o5_n70.dat', Y )


fs = 48000
blockSize = 1024

context = visr.SignalFlowContext( blockSize, fs )

renderConfigName = 'bs2051_4+5+0.xml'

# Create filtes with full paths
configPath = os.path.join( configBasePath, renderConfigName )

lc  = panning.LoudspeakerArray( configPath )

numSpeakers = lc.numberOfRegularLoudspeakers

updatePeriod = 4096
cycleDuration = 5.0

numPos = 72

# Example source trajectory (admittedly a stupid one)
azimuths = np.arange( 0, 2.0*np.pi, 2.0*np.pi/numPos )
elevations = 0.25 * np.sin(azimuths)
radius = 1.0
# Could be replaced by a sph2cart library function.
srcPos = radius * np.stack( (np.cos(azimuths)*np.cos(elevations),
                             np.sin(azimuths)*np.cos(elevations),
                             np.sin(elevations))



                          )
decodeMtx = np.loadtxt( '../data/t-design_decoder_o5_n70.dat' )
decodeMtxObj = efl.BasicMatrixFloat(decodeMtx)

calc = rcl.HoaAllradGainCalculator( context, "AllRAP", None )
calc.setup( numberOfObjectChannels=1,
           regularArrayConfig=regConfig,
           realArrayConfig=lc,
           decodeMatrix=decodeMtxObj )


flow = rrl.AudioSignalFlow( calc )

paramInput = flow.parameterReceivePort('objectInput')

gainOutput = flow.parameterSendPort("gainOutput")


gains = np.zeros( (numPos, numSpeakers ) )

for blockIdx in range(0,numPos):
    ps1 = om.PointSource(0)
    ps1.position = srcPos[:,blockIdx]
    ps1.level = 1
    ps1.channels = [ps1.objectId]

    ov = paramInput.data()
    ov.clear()
    ov.insert( ps1  )
    paramInput.swapBuffers()

    flow.process()

    hg = np.array( gainOutput.data(), copy=True )
    gains[blockIdx,:] = np.ravel( hg )


plt.figure(1)
plt.subplot(121)
plt.plot( azimuths *180/np.pi, gains, 'b.-' )
plt.title( 'AllRAP gains' )
plt.xlabel( 'azimuth (deg)' )
#plt.subplot(122)
#plt.plot( azGrid *180/np.pi, lfGains )
#axes = plt.gca()
#axes.set_xlim([-45,45])
#axes.set_ylim([0,0.9])
#plt.title( 'LF gains' )
#plt.xlabel( 'azimuth (deg)' )
axes = plt.gca()
axes.set_xlim([-45,45])
axes.set_ylim([0,0.9])
plt.show()





