#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Dec  1 10:38:25 2017

@author: andi
"""

#from realtime_multi_renderer import RealTimeMultiRenderer
from panning_comparison import RealTimePanningComparison, RealTimePanningComparisonTrajectory
from vbap_renderer import VbapRenderer, FrequencyType
from allrad_panner import AllRadPanner

import visr
import panning
import pml
import rrl
import audiointerfaces as ai

import os.path
import numpy as np
from sys import platform

# %% Create the audio configuration
# Linux and MacOS
if platform in ['linux', 'linux2','darwin']:
    # %% On Linux and MacOS X (with Jack)
    audioIfcName = 'Jack'
    optionalAudioCfg = """{ "clientname": "MultiRenderer",
                           "autoconnect" : "true",
                           "portconfig":
                               {
                               "capture":  [{ "basename":"inObj_", "externalport" : {} }],
                                            "playback": [{ "basename":"outBin_", "externalport" : {} }]
                                            }
                      }"""
elif platform == 'win32':
    # %% PortAudio on Windows
    optionalAudioCfg = """{ "hostapi": "ASIO" }"""
    audioIfcName = 'PortAudio'
else:
    print( "Unsupported platform." )


fs = 48000
blockSize = 1024
#setup = 'Salford'
setup = 'Southampton'

context = visr.SignalFlowContext( blockSize, fs )

# List of rendering configurations
if setup == 'Southampton':

    rendererConfigs = [
            { "className":VbapRenderer, "name":"VbapPlain",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0.xml"),
                        "bandType": FrequencyType.Low, "normalisationNorm": 2, "l2Panner":False } },
            { "className":VbapRenderer, "name":"VbipPlain",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0.xml"),
                        "bandType": FrequencyType.High, "normalisationNorm": 2, "l2Panner":False } },
            { "className":VbapRenderer, "name":"DualPlain",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0.xml"),
                        "bandType": FrequencyType.Dual, "normalisationNorm": [1,2], "l2Panner":False } },
            { "className":VbapRenderer, "name":"VbapDownMix",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0_mpeg-h_downmix.xml"),
                        "bandType": FrequencyType.Low, "normalisationNorm": 2, "l2Panner":False } },
            { "className":VbapRenderer, "name":"VbipDownMix",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0_mpeg-h_downmix.xml"),
                        "bandType": FrequencyType.High, "normalisationNorm": 2, "l2Panner":False } },
            { "className":VbapRenderer, "name":"DualDownMix",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0_mpeg-h_downmix.xml"),
                        "bandType": FrequencyType.Dual, "normalisationNorm": [1,2], "l2Panner":False } },
            { "className":VbapRenderer, "name":"VbapL2",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0.xml"),
                        "bandType": FrequencyType.Low, "normalisationNorm": 2, "l2Panner": True } },
            { "className":VbapRenderer, "name":"VbipL2",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0.xml"),
                        "bandType": FrequencyType.High, "normalisationNorm": 2, "l2Panner": True } },
            { "className":VbapRenderer, "name":"DualL2",
             "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0.xml"),
                        "bandType": FrequencyType.Dual, "normalisationNorm": [1, 2], "l2Panner":True } },
            { "className":AllRadPanner, "name": "AllRAD", "kwArgs": { "realArrayConfig": panning.LoudspeakerArray("../data/bs2051-4+5+0.xml"),
             "regularArrayConfig": panning.LoudspeakerArray("../data/t-design_order11_n70.xml"), "hoaOrder": 2 }  }
            ]

    configBasePath = '../data/'
    configPath = os.path.join( configBasePath, 'bs2051-4+5+0.xml')
    channelRouting = list(range(0,9))

elif setup == 'Salford':

    rendererConfigs = [
        { "className":VbapRenderer, "name":"VbapPlain",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0.xml"),
                    "bandType": FrequencyType.Low, "normalisationNorm": 2, "l2Panner":False } },
        { "className":VbapRenderer, "name":"VbipPlain",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0.xml"),
                    "bandType": FrequencyType.High, "normalisationNorm": 2, "l2Panner":False } },
        { "className":VbapRenderer, "name":"DualPlain",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0.xml"),
                    "bandType": FrequencyType.Dual, "normalisationNorm": [1,2], "l2Panner":False } },
        { "className":VbapRenderer, "name":"VbapDownMix",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0_mpeg-h_downmix.xml"),
                    "bandType": FrequencyType.Low, "normalisationNorm": 2, "l2Panner":False } },
        { "className":VbapRenderer, "name":"VbipDownMix",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0_mpeg-h_downmix.xml"),
                    "bandType": FrequencyType.High, "normalisationNorm": 2, "l2Panner":False } },
        { "className":VbapRenderer, "name":"DualDownMix",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0_mpeg-h_downmix.xml"),
                    "bandType": FrequencyType.Dual, "normalisationNorm": [1,2], "l2Panner":False } },
        { "className":VbapRenderer, "name":"VbapL2",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0.xml"),
                    "bandType": FrequencyType.Low, "normalisationNorm": 2, "l2Panner": True } },
        { "className":VbapRenderer, "name":"VbipL2",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0.xml"),
                    "bandType": FrequencyType.High, "normalisationNorm": 2, "l2Panner": True } },
        { "className":VbapRenderer, "name":"DualL2",
         "kwArgs": {"arrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0.xml"),
                    "bandType": FrequencyType.Dual, "normalisationNorm": [1, 2], "l2Panner":True } },
        { "className":AllRadPanner, "name": "AllRAD", "kwArgs": { "realArrayConfig": panning.LoudspeakerArray("../data/SalfordAudioBooth-4+5+0.xml"),
         "regularArrayConfig": panning.LoudspeakerArray("../data/t-design_order11_n70.xml"), "hoaOrder": 2 }  }
        ]

    configBasePath = '../data/'
    configPath = os.path.join( configBasePath, 'SalfordAudioBooth-4+5+0.xml')
    channelRouting = [0,1,3,5,7,12,13,14,15]

lc  = panning.LoudspeakerArray( configPath )

numberOfOutputs = lc.numberOfRegularLoudspeakers

trajectoryGenerator = True

# testSignal = None # Use a soundcard input
testSignal = os.path.join( configBasePath, 'pink_noise_1ch_48000Hz.wav' )


# Optional gain and delay adjustments

# Gains in linear scale, use 'None' for no gain adjustment
# gainAdjustment = None
gainAdjustment = np.asarray([ 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0  ])

# Delay adjustment in second (here: miliseconds plus scaling)
delayAdjustment = None
delayAdjustment = np.asarray([ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 ]) * 1.0e-3


if trajectoryGenerator:
    updatePeriod = 4096
    cycleDuration = 8.0

    numPos = int(np.ceil( cycleDuration*fs / updatePeriod ))

    # Create the same azimuthal movement at different elevations
    azimuths = np.tile( np.arange( 0, 2.0*np.pi, 2.0*np.pi/numPos ), [3,1] )

    elevations = np.tile( [0.1, 0.2, 0.3], [ numPos, 1] ).T
    radius = 1.0
    # Could be replaced by a sph2cart library function.
    positions = radius * np.stack( (np.cos(azimuths)*np.cos(elevations),
                         np.sin(azimuths)*np.cos(elevations),
                         np.sin(elevations))
                         )

    numTrajectories = positions.shape[0]

    renderer = RealTimePanningComparisonTrajectory( context, "MultiRenderer", None,
                     rendererConfigs=rendererConfigs,
                     numberOfInputs=1,
                     numberOfOutputs=numberOfOutputs,
                     trajectoryPositions=positions,
                     trajectoryUpdateSamples = updatePeriod,
                     controlReceivePort=4245,
                     controlDataType=pml.Float,
                     testSignal = testSignal,
                     useDummyInput=True,
                     outputChannelRouting = channelRouting,
                     delayAdjust = delayAdjustment,
                     gainAdjust = gainAdjustment,
                     trajectoryControlInputType = pml.Integer,
                     trajectoryControlPortNumber = 4246,
                     initialTrajectoryIndex = 0
                     )
else:
    renderer = RealTimePanningComparison( context, "MultiRenderer", None,
                 rendererConfigs=rendererConfigs,
                 numberOfInputs=1,
                 numberOfOutputs=numberOfOutputs,
                 interpolationSteps = blockSize,
                 sceneReceivePort=4242,
                 controlReceivePort=4245,
                 outputChannelRouting = channelRouting,
                 delayAdjust = delayAdjustment,
                 gainAdjust = gainAdjustment
                 )

flowCheck, messages = rrl.checkConnectionIntegrity( renderer )
if not flowCheck:
    print( "Integrity check for multirenderer failed: %s" % messages )
    # sys.exit()

flow = rrl.AudioSignalFlow( renderer )

aiConfig = ai.AudioInterface.Configuration( flow.numberOfCaptureChannels,
                                           flow.numberOfPlaybackChannels,
                                           fs,
                                           blockSize )

aIfc = ai.AudioInterfaceFactory.create( audioIfcName, aiConfig, optionalAudioCfg )

aIfc.registerCallback( flow )

aIfc.start()

print( "Rendering started." )

#
# time.sleep(1)
i = input("Enter text (or Enter to quit): ")
if not i:
    aIfc.stop()
    aIfc.unregisterCallback()
    del aIfc


