     # -*- coding: utf-8 -*-
"""
Created on Tue Feb 14 15:59:11 2017

@author: af5u13
"""

import visr
import rcl
import rrl
import objectmodel as om
import panning


import numpy as np;
import matplotlib.pyplot as plt
import sys

# from vbap_l2_renderer import VbapL2Renderer
from vbap_renderer import VbapRenderer, FrequencyType

# from vbap_l2_panner import VbapL2Panner


def sph2cart(az,el,r):
    az,el,r = np.broadcast_arrays( az, el, r )
    x = r*np.cos(az)*np.cos(el)
    y = r*np.sin(az)*np.cos(el)
    z = r*np.sin(el)
    return x,y,z

bs = 128
samplingFrequency = 48000

numBlocks = 128

numObjects = 1

signalLength = bs * numBlocks
t = 1.0/samplingFrequency * np.arange(0,signalLength)

ctxt = visr.SignalFlowContext( bs, samplingFrequency)

lc = panning.LoudspeakerArray( '../data/bs2051-4+5+0.xml' )

numOutputChannels = lc.numberOfRegularLoudspeakers

#             numberOfInputs,
#             arrayConfig,
#             interpolationSteps,
#             bandType = FrequencyType.Low,
#             normalisationNorm = None,   # Default value
#             l2Panner = False,
#             filterBank = None

rendererVbap = VbapRenderer( ctxt, 'renderer', None,
                            numberOfInputs=numObjects,
                            arrayConfig=lc,
                            interpolationSteps=bs,
                            bandType=FrequencyType.Dual,
                            normalisationNorm=2,
                            l2Panner=True )
#renderer1 = VbapL2Renderer( ctxt, 'renderer', None, numObjects, lspArray=lc )

#flow = rrl.AudioSignalFlow( renderer1 )

res, msg = rrl.checkConnectionIntegrity( rendererVbap )
if not res:
    print( msg )
    sys.exit()

flowVbap = rrl.AudioSignalFlow( rendererVbap )

# paramInput = flow.parameterReceivePort('objects')

paramInputVbap = flowVbap.parameterReceivePort('objects')

az = np.linspace( 0, 2.0*np.pi, numBlocks )
el = 10.0 * np.pi/180.0
r = 1.0

inSig = np.zeros( (numObjects, signalLength ), dtype=np.float32 )
inSig[0,:] = 0.75*np.sin( 2.0*np.pi*88 * t )
#outSig = np.zeros( (numOutputChannels, signalLength ), dtype=np.float32 )

#for bi in range(0,numBlocks):
#    # x,y,z = sph2cart( az[bi], el, r )
#    ps1 = om.PointSource(0)
#    # ps1.position = [x,y,z]
#    ps1.position = sph2cart( az[bi], el, r )
#    ps1.channels = [0]; ps1.level=1.0
#
#    paramInput.data().set( [ps1] )
#    paramInput.swapBuffers()
#    outSig[:,bi*bs:(bi+1)*bs] = flow.process( inSig[:, bi*bs:(bi+1)*bs] )

outSigVbap = np.zeros( (numOutputChannels, signalLength ), dtype=np.float32 )

for bi in range(0,numBlocks):
    ps1 = om.PointSource(0)
    ps1.position = sph2cart( az[bi], el, r )
    ps1.channels = [0]; ps1.level=1.0

    paramInputVbap.data().set( [ps1] )
    paramInputVbap.swapBuffers()
    outSigVbap[:,bi*bs:(bi+1)*bs] = flowVbap.process( inSig[:, bi*bs:(bi+1)*bs] )


plt.figure()
plt.plot( t, outSigVbap[7,:], linestyle='--', color=(1.0,0.5,0.5) )
plt.xlabel( 'time [s]' )
plt.ylabel( 'Amplitude' )
plt.tight_layout()
# plt.gca().set_aspect( 3 )
#plt.savefig( '../../figures/offline_simulation_audio_signal.pdf' )
#audioData = np.concatenate( (np.reshape( t, (1,-1) ), outSigVbap[7:8,:], outSig[7:8,:] ), axis=0 )
#np.savetxt( '../../data/offline_simulation_audio.dat', audioData[:,::16].T )

# %% Visualise panning gains
#from helper.vectorFunctions import angleDifference
#from helper.panningVectorMetrics import re
#from helper.baseTrigFunctions import rad2deg
#
#
#pannerVbap = rcl.PanningCalculator( ctxt, 'renderer', None, numObjects, lc )
#pannerVbapL2 = VbapL2Panner( ctxt, 'renderer', None, numObjects, lc )
#
#flowVbapPanner = rrl.AudioSignalFlow( pannerVbap )
#flowVbapL2Panner = rrl.AudioSignalFlow( pannerVbapL2 )
#
#paramInputVbapPanner = flowVbapPanner.parameterReceivePort('objectVectorInput')
#paramInputVbapL2Panner = flowVbapL2Panner.parameterReceivePort('objects')
#paramOutputVbapPanner = flowVbapPanner.parameterSendPort('gainOutput')
#paramOutputVbapL2Panner = flowVbapL2Panner.parameterSendPort('gains')
#
#gainsVbap = np.zeros( (numOutputChannels,numBlocks), dtype=np.float32 )
#gainsVbapL2 = np.zeros( (numOutputChannels,numBlocks), dtype=np.float32 )
#
#for bi in range(0,numBlocks):
#    ps1 = om.PointSource(0)
#    ps1.position = sph2cart( az[bi], el, r )
#    ps1.channels = [0]; ps1.level=1.0
#
#    paramInputVbapPanner.data().set( [ps1] )
#    paramInputVbapPanner.swapBuffers()
#    flowVbapPanner.process()
#    g = np.squeeze( np.array(paramOutputVbapPanner.data() ) )
#    gainsVbap[:,bi] = g
#
#    paramInputVbapL2Panner.data().set( [ps1] )
#    paramInputVbapL2Panner.swapBuffers()
#    flowVbapL2Panner.process()
#    gainsVbapL2[:,bi] = np.squeeze( paramOutputVbapL2Panner.data() )
#
## %%
#
#azDeg = rad2deg( az )
#plt.figure()
#plt.plot( azDeg, gainsVbap[7,:], linestyle='-', color=(0.5,0.5,0.5) )
#plt.plot( azDeg, gainsVbap[8,:], linestyle='--', color=(0.5,0.5,0.5) )
#plt.plot( azDeg, gainsVbapL2[7,:], linestyle='-', color='red' )
#plt.plot( azDeg, gainsVbapL2[8,:], linestyle='--', color='red' )
#plt.xlabel( 'Source azimuth [deg]' )
#plt.ylabel( 'Loudspeaker gain' )
#plt.tight_layout()
#plt.gca().set_aspect( 3 )
#plt.savefig( '../../figures/offline_simulation_gains.pdf' )
#gainData = np.concatenate( (np.reshape( azDeg, (1,-1) ), gainsVbap[7:9,:], gainsVbapL2[7:9,:] ), axis=0 )
#np.savetxt( '../../data/offline_simulation_gains.dat', gainData.T )
#
## %% Compute energy vector difference
#
#L = lc.positions()
#
#pDes = np.stack( sph2cart( az, el, r ), axis=-1 )
#
#reVbap, reDirVbap, reMagVbap = re( gainsVbap.T, L.T  )
#reVbapL2, reDirVbapL2, reMagVbapL2 = re( gainsVbapL2.T, L.T  )
#
#reAngleDiff = rad2deg(angleDifference( pDes, reDirVbap.T) )
#reAngleDiffL2 = rad2deg(angleDifference( pDes, reDirVbapL2.T) )
#
#plt.figure()
#plt.plot( azDeg, reAngleDiff, linestyle='-', color=(0.5,0.5,0.5) )
#plt.plot( azDeg, reAngleDiffL2, linestyle='-', color='red' )
#plt.xlabel( 'Source azimuth [deg]' )
#plt.ylabel( 'Energy direction error [deg]')
#plt.tight_layout()
#plt.gca().set_aspect( 3 )
#plt.savefig( '../../figures/offline_simulation_energy_direction_difference.pdf' )
#reData = np.concatenate( (np.reshape( azDeg, (1,-1) ), np.reshape(reAngleDiff, (1,-1)), np.reshape(reAngleDiffL2, (1,-1))), axis=0 )
#np.savetxt( '../../data/offline_simulation_energy_direction_difference.dat', reData.T )
#
#
