# -*- coding: utf-8 -*-
"""
Created on Sun Feb 26 16:41:01 2017

@author: andi
"""

import visr

import math
import numpy as np
import scipy.io.wavfile as wavfile


class WavPlayback( visr.AtomicComponent ):
    def __init__( self, context, name, parent,
                 audioFile,
                 numberOfChannels = None,
                 ):
        super( WavPlayback, self ).__init__( context, name, parent )

        fs, sig = wavfile.read( audioFile )
        if fs != context.samplingFrequency:
            self.status( visr.StatusMessage.Warning, "Sampling frequency of audio file (%d Hz) differs from system samopling frequency (%d Hz)." % (fs, context.samplingFrequency))

        if np.issubdtype( sig.dtype , np.integer):
            scaleFactor = 1.0/float(np.iinfo(sig.dtype).max)
            sigConverted = scaleFactor * np.asarray( sig.T, dtype=np.float32 )
        else:
            sigConverted = np.asarray( sig.T, dtype=np.float32 )
        if len( sigConverted.shape ) == 1: # Mono signal
            sigConverted = np.reshape( sigConverted, (1, sigConverted.shape[0]) )

        if numberOfChannels is None:
            numberOfChannels = sigConverted.shape[0]
#        elif numberOfChannels != sigConverted.shape[0]: # For the moment we demand equality.
#            self.status( visr.StatusMessage.Error, "Number of wav channels (%d) differs from 'numberOfChannels' parameter (%d)." % (sigConverted.shape[0], numberOfChannels) )
        elif numberOfChannels < sigConverted.shape[0]: # For the moment we demand equality.
            sigConverted = sigConverted[0:numberOfChannels,:]
        elif numberOfChannels > sigConverted.shape[0]: # For the moment we demand equality.
            sigConverted = np.concatenate( (sigConverted, np.zeros( (numberOfChannels - sigConverted.shape[0],
                                                                     sigConverted.shape[1]), dtype=sigConverted.dtype )), axis=0 )

        # Periodically repeat signal at least to the length of a period
        if sigConverted.shape[-1] < context.period:
            numReps = int( math.ceil( context.period / sigConverted.shape[-1] ))
            sigConverted = np.repeat( sigConverted, numReps, axis=-1 )

        padding = sigConverted.shape[-1] - (sigConverted.shape[-1] // context.period )*context.period
        self.signal = np.concatenate( (sigConverted, sigConverted[...,0:padding] ), axis=-1 )

        self.output = visr.AudioOutputFloat( "out", self, numberOfChannels )

        self.blockSize = context.period
        self.startLimit = self.signal.shape[-1] - context.period
        self.readCounter = int(0)

    def process( self ):
        try:
            val=  self.signal[:,self.readCounter:self.readCounter+self.blockSize]
            self.output.set( val )
            self.readCounter += self.blockSize
            if self.readCounter >= self.startLimit:
                self.readCounter -= self.startLimit
        except Exception as ex:
            print( "WavPlayback::process(): exception %s." % str(ex) )