% close all;
clear variables; clc;

%% Settings

% Speaker angles and triplets

% % Audio booth (near) cube
% phiLS = [45:90:359 45:90:359];
% thetaLS = [30*ones(1,4) -30*ones(1,4)];
% % LStriplets = [1 2 5;2 6 5;2 3 6;3 7 6;3 4 7;4 8 7;4 1 8;1 5 8; ...
% %     1 3 2;1 4 3;5 6 7;7 8 5];

% % Audio booth full
% phiLS = [0:45:359 45:90:359 45:90:359];
% thetaLS = [zeros(1,8) 30*ones(1,4) -30*ones(1,4)];

% 0+5+2 (with virtual below)
% phiLS = [0 30 110 -110 -30 30 -30 0];
% thetaLS = [zeros(1,5) 30*ones(1,2) -90];

% 0+5+4 (with virtual below)
phiLS = [0 30 110 -110 -30 30 110 -110 -30 0];
thetaLS = [zeros(1,5) 30*ones(1,4) -90];

% % 22.0 (3+10+9)
% phiLS = [0:45:359 30 -30 0:45:359 0 45 0 -45];
% thetaLS = [zeros(1,10) 30*ones(1,8) 90 -30*ones(1,3)];
% % phiLS(9:10) = [];   thetaLS(9:10) = [];

LStriplets = convhull(cosd(phiLS).*cosd(thetaLS), ...
    sind(phiLS).*cosd(thetaLS),sind(thetaLS));

%%

% Vector in directions of speakers
vLS = [cosd(phiLS).*cosd(thetaLS);sind(phiLS).*cosd(thetaLS); ...
    sind(thetaLS)];

% Number of points for maps
Nphi = 361;
Ntheta = 181;
Nmap = Nphi*Ntheta;
phi = ones(Ntheta,1)*linspace(-180,180,Nphi);
theta = linspace(90,-90,Ntheta).'*ones(1,Nphi);

% Map unit vectors
uv = [cosd(phi(:)).*cosd(theta(:)) sind(phi(:)).*cosd(theta(:))  ...
    sind(theta(:))].';

% Number of triplets to test
Ntriplet = size(LStriplets,1);

% To store results
phiV = zeros(1,Nmap);   thetaV = zeros(1,Nmap);
phiE = zeros(1,Nmap);   thetaE = zeros(1,Nmap);
sigmaE = zeros(1,Nmap); sigmaV = zeros(1,Nmap);
E = zeros(1,Nmap);     V = zeros(1,Nmap);
rE = zeros(3,Nmap);     rV = zeros(3,Nmap);
VIangle = zeros(1,Nmap);

for i = 1:Nmap
    
    % Unit vector to test
    uvi = uv(:,i);
    
    for j = 1:Ntriplet
        
        % j-th loudspeaker triplet
        vLSj = vLS(:,LStriplets(j,:));
        
        % VBAP gains
        VBAPg = vLSj\uvi;
        valid = all(VBAPg>=-1e-10);
        
        if valid
            
            % Normalise gains
            VBAPg = VBAPg./(ones(3,1)*sqrt(sum(VBAPg.^2,1)));
%             VBAPg = real(sqrt(VBAPg));
%             VBAPg = VBAPg./(ones(3,1)*sqrt(sum(VBAPg.^2,1)));
            % Source magnitude (pressure and energy summations)
            Vi = sum(VBAPg);
            Ei = sum(VBAPg.^2);
            
            % Resulting velocity and intensity rV and rE vectors
            rVi = vLSj*VBAPg;     rVi = rVi./Vi;
            rEi = vLSj*VBAPg.^2;  rEi = rEi./Ei;
            
            % Resulting angles (velocity angles should clearly match input)
            phiVi = atan2(rVi(2,:),rVi(1,:))*180/pi;
            thetaVi = atan2(rVi(3,:),sqrt(sum(rVi(1:2,:).^2)))*180/pi;
            phiIi = atan2(rEi(2,:),rEi(1,:))*180/pi;
            thetaIi = atan2(rEi(3,:),sqrt(sum(rEi(1:2,:).^2)))*180/pi;
            
            % Angle between velocity and intensity vector
            VIanglei = acosd(rVi.'*rEi./sqrt(sum(rEi.^2)*sum(rVi.^2)));
            
%             % Apparent source extent (spherical cap)
%             sigmaVi = 2*acos(sqrt(sum(rVi.^2,1)))*180/pi;
%             sigmaEi = 2*acos(sqrt(sum(rEi.^2,1)))*180/pi;
            % Apparent source extent (Epain, Jin, Zotter, 2014)
            sigmaVi = 2*acos(2*sqrt(sum(rVi.^2,1))-1)*180/pi;
            sigmaEi = 2*acos(2*sqrt(sum(rEi.^2,1))-1)*180/pi;
            
        end
        
    end
    
    % Store results
    phiV(i)     = phiVi;
    thetaV(i)   = thetaVi;
    phiE(i)     = phiIi;
    thetaE(i)   = thetaIi;
    V(i)        = Vi;
    E(i)        = Ei;
    rE(:,i)     = rEi;
    rV(:,i)     = rVi;
    sigmaV(i)   = real(sigmaVi);
    sigmaE(i)   = real(sigmaEi);
    VIangle(i)  = real(VIanglei);
    
end

%%

hf(1) = figure;
imagesc(phi(1,:),theta(:,1),reshape(VIangle,[Ntheta Nphi]))
title('VBAP - intensity vector angular error')

hf(2) = figure;
imagesc(phi(1,:),theta(:,1),reshape(20*log10(V./E),[Ntheta Nphi]))
title('VBAP - low frequency EQ error')

hf(3) = figure;
imagesc(phi(1,:),theta(:,1),reshape(sigmaV-sigmaE,[Ntheta Nphi]))
title('VBAP - low and high frequency deviation in ''extent''')

for k = 1:length(hf)
    figure(hf(k))
    hold on
    plot(mod(phiLS+180,360)-180,mod(thetaLS+180,360)-180,'wo','linewidth',2)
    for i = 1:Ntriplet
        for j = 1:3
            phij1 = phiLS(LStriplets(i,j));
            thetaj1 = thetaLS(LStriplets(i,j));
            phij2 = phiLS(LStriplets(i,mod(j,3)+1));
            thetaj2 = thetaLS(LStriplets(i,mod(j,3)+1));
            vj1 = [cosd(phij1).*cosd(thetaj1); ...
                sind(phij1).*cosd(thetaj1);sind(thetaj1)];
            vj2 = [cosd(phij2).*cosd(thetaj2); ...
                sind(phij2).*cosd(thetaj2);sind(thetaj2)];
            vj = vj1*ones(1,100)+(vj2-vj1)*linspace(0,1,100);
            phij = atan2(vj(2,:),vj(1,:))*180/pi;
            thetaj = atan2(vj(3,:),sqrt(sum(vj(1:2,:).^2)))*180/pi;
            test = abs(diff(gradient(thetaj)))>2;
            if any(test)
                ntest = find(test);
            else
                ntest = [];
            end
            test = abs(diff(phij))>180;
            if any(test)
                ntest = unique([ntest find(test)]);
            end
            if ~isempty(ntest)
                plot(phij(1:ntest),thetaj(1:ntest),'w:','linewidth',2)
                plot(phij(ntest+1:end),thetaj(ntest+1:end),'w:', ...
                    'linewidth',2)
            else
                plot(phij,thetaj,'w:','linewidth',2)
            end
        end
    end
    colorbar
    set(gca,'ydir','normal')
    set(gca,'xdir','reverse')
    axis equal
    axis([-180 180 -90 90])
    % axis([-50 50 -35 35])
    xlabel('Azimuth, \phi (degrees)')
    ylabel('Elevation, \theta (degrees)')
    xlim = get(gca,'xlim');
    ylim = get(gca,'ylim');
    set(gca,'xtick',-180:45:180,'xticklabel',{'-180' '-135' '-90' '-45' ...
        '0' '+45' '+90' '+135' '+180'})
    set(gca,'ytick',-90:30:90,'yticklabel',{'-90' '-60' '-30' '0' '+30' ...
        '+60' '+90'})
end

figure(hf(1))
ht1 = text(xlim(2)+0.08*(xlim(2)-xlim(1)),ylim(1)+0.5*(ylim(2)-ylim(1)), ...
    'Degrees');
set(ht1,'rotation',90,'horizontalalignment','center')
figure(hf(2))
ht2 = text(xlim(2)+0.08*(xlim(2)-xlim(1)),ylim(1)+0.5*(ylim(2)-ylim(1)), ...
    'dB');
set(ht2,'rotation',90,'horizontalalignment','center')
figure(hf(3))
ht3 = text(xlim(2)+0.08*(xlim(2)-xlim(1)),ylim(1)+0.5*(ylim(2)-ylim(1)), ...
    'Degrees');
set(ht3,'rotation',90,'horizontalalignment','center')

%% For paper

figure(hf(1))
axis([-120 120 -5 90])
title('')
caxis([0 25])
set(hf(1),'position',[385 239 855 739])
plot([-180 180],[10 10],'color',0.7*ones(1,3),'linestyle','--','linewidth',4)
set(ht1,'position',[-150 40 0])
set(ht1,'fontsize',16)
hc = get(hf(1),'children');
for i = 1:length(hc)
    set(hc(i),'fontsize',16)
end

figure(hf(2))
axis([-120 120 -5 90])
title('')
caxis([0 5])
set(hf(2),'position',[385 239 855 739])
plot([-180 180],[30 30],'color',0.7*ones(1,3),'linestyle','--','linewidth',4)
set(ht2,'position',[-150 40 0])
set(ht2,'fontsize',14)
hc = get(hf(2),'children');
for i = 1:length(hc)
    set(hc(i),'fontsize',16)
end

figure(hf(3))
axis([-120 120 -5 90])
title('')
caxis([0 90])
set(hf(3),'position',[385 239 855 739])
plot(-7.5,5,'+','color',0.7*ones(1,3),'markersize',12,'linewidth',4)
set(ht3,'position',[-150 40 0])
set(ht3,'fontsize',16)
hc = get(hf(3),'children');
for i = 1:length(hc)
    set(hc(i),'fontsize',16)
end

return

%% Plot maps as spheres

close all;

scrsz = get(0,'ScreenSize');

[xx,yy,zz] = sph2cart(phi*pi/180,theta*pi/180,ones(size(phi)));
TRI = delaunay(xx,yy,zz);

hf(1) = figure;
set(hf(1),'color',[1 1 1])
surf(xx,yy,zz,reshape(VIangle,[Ntheta Nphi]))
title('VBAP - intensity vector angular error')

hf(2) = figure;
set(hf(2),'color',[1 1 1])
surf(xx,yy,zz,reshape(20*log10(V./E),[Ntheta Nphi]))
title('VBAP - low frequency EQ error')

hf(3) = figure;
set(hf(3),'color',[1 1 1])
surf(xx,yy,zz,reshape(sigmaV-sigmaE,[Ntheta Nphi]))
title('VBAP - low and high frequency deviation in ''extent''')

for k = 1:length(hf)
    figure(hf(k))
    hold on
    shading interp
    axis off
    axis equal
    % view([0 0 1])
    axis vis3d
    view([1 1 1])
    colorbar
    plot3(1.5*[-1 1],[0 0],[0 0],'k-')
    plot3([0 0],1.5*[-1 1],[0 0],'k-')
    plot3([0 0],[0 0],1.5*[-1 1],'k-')
    text(1.6,0,0,'+x'); text(-1.6,0,0,'-x');
    text(0,1.6,0,'+y'); text(0,-1.6,0,'-y')
    text(0,0,1.6,'+z'); text(0,0,-1.6,'-z')
    for i = 1:Ntriplet
        for j = 1:3
            phij1 = phiLS(LStriplets(i,j));
            thetaj1 = thetaLS(LStriplets(i,j));
            phij2 = phiLS(LStriplets(i,mod(j,3)+1));
            thetaj2 = thetaLS(LStriplets(i,mod(j,3)+1));
            vj1 = [cosd(phij1).*cosd(thetaj1); ...
                sind(phij1).*cosd(thetaj1);sind(thetaj1)];
            vj2 = [cosd(phij2).*cosd(thetaj2); ...
                sind(phij2).*cosd(thetaj2);sind(thetaj2)];
            vj = vj1*ones(1,100)+(vj2-vj1)*linspace(0,1,100);
            vj = 1.01*vj./(ones(3,1)*sqrt(sum(vj.^2,1)));
            plot3(vj(1,:),vj(2,:),vj(3,:),'w:','linewidth',2)
        end
    end
    set(gcf,'position',scrsz);
end

figure(hf(1))
annotation(hf(1),'textbox',[0.92 0.50 0.06 0.06],'String',{'Degrees'},...
    'verticalalignment','middle','horizontalalignment','center',...
    'fitboxtotext','on','linestyle','none');

figure(hf(2))
annotation(hf(2),'textbox',[0.92 0.50 0.06 0.06],'String',{'dB'},...
    'verticalalignment','middle','horizontalalignment','center',...
    'fitboxtotext','on','linestyle','none');

figure(hf(3))
annotation(hf(3),'textbox',[0.92 0.50 0.06 0.06],'String',{'Degrees'},...
    'verticalalignment','middle','horizontalalignment','center',...
    'fitboxtotext','on','linestyle','none');

%%
return
%%

figure
pos = get(gcf,'position');
close(gcf)

video_name = {'Localisation','EQ','Extent'};

Nrotate = 360;
view_angles = 45+360*(0:Nrotate-1).'./Nrotate;
rotate_view = [cosd(view_angles) sind(view_angles) ones(Nrotate,1)*2^-.5];

for j = 1:3
    vidObj = VideoWriter(video_name{j},'MPEG-4');
    open(vidObj);
    figure(hf(j))
    title('')
    set(hf(j),'position',pos);
    for i = 1:size(rotate_view,1)
        view(rotate_view(i,:))
        writeVideo(vidObj,getframe(gcf));
    end
    close(vidObj)
end



